'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fetch = require('./adapters/fetch.js');

var _fetch2 = _interopRequireDefault(_fetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var defaultSettings = {
	adapter: _fetch2.default,
	primaryKey: 'id',
	actionTypesSuffixPlural: undefined,
	json: false,
	headers: [],
	getters: {},
	responseType: undefined
};

var ERROR_SUFFIX = '_ERROR';

var CrudActionCreators = function () {
	function CrudActionCreators(url, actionTypesSuffix) {
		var settings = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

		_classCallCheck(this, CrudActionCreators);

		this.url = url;
		this.primaryKey = settings.primaryKey || defaultSettings.primaryKey;
		this.adapter = settings.adapter || defaultSettings.adapter;
		this.json = settings.json;
		this.headers = settings.headers || defaultSettings.headers;
		this.getters = settings.getters || defaultSettings.getters;
		this.responseType = settings.responseType || defaultSettings.responseType;
		this.additionalAdapterSettings = settings.additionalAdapterSettings;

		var actionTypesSuffixPl = settings.actionTypesSuffixPlural || actionTypesSuffix + 'S';

		this.actionTypes = {
			request: 'REQUEST_' + actionTypesSuffixPl,
			receive: 'RECEIVED_' + actionTypesSuffixPl,
			receiveError: 'RECEIVED_' + actionTypesSuffixPl + ERROR_SUFFIX,
			saving: 'SAVING_' + actionTypesSuffix,
			saved: 'SAVED_' + actionTypesSuffix,
			saveError: 'SAVE_' + actionTypesSuffix + ERROR_SUFFIX,
			requestOne: 'REQUEST_' + actionTypesSuffix,
			receiveOne: 'RECEIVED_' + actionTypesSuffix,
			receiveOneError: 'RECEIVED_' + actionTypesSuffix + ERROR_SUFFIX,
			deleting: 'DELETING_' + actionTypesSuffix,
			deleted: 'DELETED_' + actionTypesSuffix,
			deleteError: 'DELETE_' + actionTypesSuffix + ERROR_SUFFIX
		};

		this.requestList = this.requestList.bind(this);
		this.receiveList = this.receiveList.bind(this);
		this.receiveListError = this.receiveListError.bind(this);
		this.saving = this.saving.bind(this);
		this.saved = this.saved.bind(this);
		this.saveError = this.saveError.bind(this);
		this.requestOne = this.requestOne.bind(this);
		this.receiveOne = this.receiveOne.bind(this);
		this.receiveOneError = this.receiveOneError.bind(this);
		this.deleting = this.deleting.bind(this);
		this.deleted = this.deleted.bind(this);
		this.deleteError = this.deleteError.bind(this);
	}

	_createClass(CrudActionCreators, [{
		key: '_getHeaders',
		value: function _getHeaders() {
			if (_typeof(this.headers) == 'object') {
				return this.headers;
			}

			if (typeof this.headers == 'function') {
				return this.headers();
			}

			return {};
		}
	}, {
		key: 'requestList',
		value: function requestList(params) {
			return {
				type: this.actionTypes.request,
				params: params
			};
		}
	}, {
		key: 'receiveList',
		value: function receiveList(params, data) {
			return {
				type: this.actionTypes.receive,
				data: this.getters.getList ? this.getters.getList(data) : data,
				params: params
			};
		}
	}, {
		key: 'receiveListError',
		value: function receiveListError(params, data, error) {
			return {
				type: this.actionTypes.receiveError,
				data: data,
				error: error,
				params: params
			};
		}
	}, {
		key: 'fetchList',
		value: function fetchList(params) {
			return this.adapter(_extends({
				url: this.url,
				params: params,
				headers: this._getHeaders(),
				requestAction: this.requestList,
				successAction: this.receiveList.bind(null, params),
				failureAction: this.receiveListError.bind(null, params)
			}, this.additionalAdapterSettings));
		}
	}, {
		key: 'saving',
		value: function saving(id) {
			return {
				type: this.actionTypes.saving,
				id: id
			};
		}
	}, {
		key: 'saved',
		value: function saved(id, data) {
			return {
				type: this.actionTypes.saved,
				data: data,
				id: id
			};
		}
	}, {
		key: 'saveError',
		value: function saveError(id, data, error) {
			return {
				type: this.actionTypes.saveError,
				id: id,
				data: data,
				error: error
			};
		}
	}, {
		key: 'save',
		value: function save(data, id, method) {
			if (id) {
				data[this.primaryKey] = id;
			}

			return this.adapter(_extends({
				url: id ? this.url + '/' + id : this.url,
				params: data,
				method: method || (id ? 'PUT' : 'POST'),
				json: this.json,
				headers: this._getHeaders(),
				requestAction: this.saving,
				successAction: this.saved.bind(null, id),
				failureAction: this.saveError.bind(null, id)
			}, this.additionalAdapterSettings));
		}
	}, {
		key: 'requestOne',
		value: function requestOne(id) {
			return {
				type: this.actionTypes.requestOne,
				id: id
			};
		}
	}, {
		key: 'receiveOne',
		value: function receiveOne(id, data) {
			return {
				type: this.actionTypes.receiveOne,
				id: id,
				data: this.getters.getOne ? this.getters.getOne(data) : data
			};
		}
	}, {
		key: 'receiveOneError',
		value: function receiveOneError(id, data, error) {
			return {
				type: this.actionTypes.receiveOneError,
				id: id,
				error: error,
				data: data
			};
		}
	}, {
		key: 'fetchOne',
		value: function fetchOne(id) {
			return this.adapter(_extends({
				url: this.url + '/' + id,
				headers: this._getHeaders,
				requestAction: this.requestOne,
				successAction: this.receiveOne.bind(null, id),
				failureAction: this.receiveOneError.bind(null, id)
			}, this.additionalAdapterSettings));
		}
	}, {
		key: 'deleting',
		value: function deleting(id) {
			return {
				type: this.actionTypes.deleting,
				id: id
			};
		}
	}, {
		key: 'deleted',
		value: function deleted(id, data) {
			return {
				type: this.actionTypes.deleted,
				data: data,
				id: id
			};
		}
	}, {
		key: 'deleteError',
		value: function deleteError(id, data, error) {
			return {
				type: this.actionTypes.deleteError,
				data: data,
				error: error,
				id: id
			};
		}
	}, {
		key: 'delete',
		value: function _delete(id) {
			return this.adapter(_extends({
				url: this.url + '/' + id,
				method: 'DELETE',
				headers: this._getHeaders,
				requestAction: this.deleting,
				successAction: this.deleted.bind(null, id),
				failureAction: this.deleteError.bind(null, id)
			}, this.additionalAdapterSettings));
		}
	}]);

	return CrudActionCreators;
}();

exports.default = CrudActionCreators;