'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var requestAction = _ref.requestAction,
	    successAction = _ref.successAction,
	    failureAction = _ref.failureAction,
	    method = _ref.method,
	    url = _ref.url,
	    params = _ref.params,
	    json = _ref.json,
	    headers = _ref.headers,
	    responseType = _ref.responseType;

	return function (dispatch) {
		if (requestAction) {
			dispatch(requestAction(params));
		}

		var hasBody = methodsWithBody.indexOf(method) >= 0;

		var urlFinal = url;
		var paramKeys = params ? Object.keys(params) : [];

		if (!hasBody && paramKeys.length) {
			urlFinal += '?';
			paramKeys.forEach(function (paramKey, index) {
				if (index) {
					urlFinal += '&';
				}

				urlFinal += paramKey + '=' + params[paramKey];
			});
		}

		return fetch(urlFinal, {
			method: method,
			headers: new Headers(headers),
			data: hasBody ? json ? JSON.stringify(params) : params : undefined
		}).then(function (response) {
			var responseTypeFinal = response[responseType] == 'function' ? responseType : 'json';

			if (responseTypeFinal == 'json' && response.body._readableState.length == 0) {
				responseTypeFinal = 'text';
			}

			var dataPromise = response[responseTypeFinal]();

			dataPromise.then(function (data) {
				if (response.ok) {
					if (successAction) {
						dispatch(successAction(data));
					}
				} else if (failureAction) {
					dispatch(failureAction(data, response.statusText));
				}
			});

			return dataPromise;
		}).catch(function (error) {
			console.log(error);
			if (failureAction) {
				dispatch(failureAction(null, error || 'Network error'));
			}

			return null;
		});
	};
};

var _redux = require('redux');

var _redux2 = _interopRequireDefault(_redux);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var methodsWithBody = ['POST', 'PUT', 'PATCH'];