'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (actionCreators) {
	var availableCrudActions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'CRUD';
	var resetAllActionType = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'RESET_ALL_DATA';

	var _actionHandlers;

	var mergeDataChanges = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
	var initialDataState = arguments[4];

	var initialState = _extends({}, readInitialState, {
		data: initialDataState || []
	});

	var hasSave = availableCrudActions.indexOf('C') >= 0 || availableCrudActions.indexOf('U') >= 0;
	var hasDelete = availableCrudActions.indexOf('D') >= 0;

	if (hasSave) {
		initialState = _extends({}, initialState, saveInitialState);
	}

	if (hasDelete) {
		initialState = _extends({}, initialState, deleteInitialState);
	}

	var actionTypes = actionCreators.actionTypes;

	var actionHandlers = (_actionHandlers = {}, _defineProperty(_actionHandlers, actionTypes.request, request), _defineProperty(_actionHandlers, actionTypes.receive, receive), _defineProperty(_actionHandlers, actionTypes.receiveError, receiveError), _defineProperty(_actionHandlers, actionTypes.requestOne, requestOne), _defineProperty(_actionHandlers, actionTypes.receiveOne, receiveOne), _defineProperty(_actionHandlers, actionTypes.receiveOneError, receiveOneError), _actionHandlers);

	if (hasSave) {
		actionHandlers[actionTypes.saving] = saving;
		actionHandlers[actionTypes.saved] = mergeDataChanges ? savedWithMerge : saved;
		actionHandlers[actionTypes.saveError] = saveError;
	}

	if (hasDelete) {
		actionHandlers[actionTypes.deleting] = deleting;
		actionHandlers[actionTypes.deleted] = mergeDataChanges ? deletedWithMerge : deleted;
		actionHandlers[actionTypes.deleteError] = deleteError;
	}

	return function () {
		var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
		var action = arguments[1];

		if (action.type === resetAllActionType) {
			return initialState;
		} else if (!action || !action.type || !actionHandlers[action.type]) {
			return state;
		}

		return actionHandlers[action.type](state, action);
	};
};

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var readInitialState = {
	data: [],
	listError: undefined,
	params: undefined,
	listLoading: false,
	oneLoading: false,
	currentId: undefined,
	currentData: undefined,
	oneError: undefined
};

var saveInitialState = {
	isSaving: false,
	savedData: undefined,
	saveError: undefined
};

var deleteInitialState = {
	isDeleting: false,
	deleteError: undefined
};

function request(state, action) {
	return _extends({}, state, {
		listLoading: true,
		params: action.params,
		data: readInitialState.data,
		listError: readInitialState.listError
	});
}

function receive(state, action) {
	return _extends({}, state, {
		data: action.data,
		listLoading: false
	});
}

function receiveError(state, action) {
	return _extends({}, state, {
		listError: action.error,
		listLoading: false
	});
}

function requestOne(state, action) {
	return _extends({}, state, {
		currentId: readInitialState.currentId,
		currentData: readInitialState.currentData,
		oneLoading: true,
		oneError: readInitialState.oneError
	});
}

function receiveOne(state, action) {
	return _extends({}, state, {
		currentId: action.id,
		currentData: action.data,
		oneLoading: false
	});
}

function receiveOneError(state, action) {
	return _extends({}, state, {
		oneLoading: false,
		oneError: action.error
	});
}

function saving(state, action) {
	return _extends({}, state, {
		isSaving: true,
		savedData: saveInitialState.savedData,
		saveError: saveInitialState.error
	});
}

function saved(state, action) {
	return _extends({}, state, {
		savedData: action.data,
		isSaving: false
	});
}

function savedWithMerge(state, action) {
	if (action.id) {
		var data = state.data.map(function (item) {
			if (item.id == action.id) {
				return action.data;
			}
			return item;
		});
		return _extends({}, state, {
			isSaving: false,
			savedData: action.data,
			data: data
		});
	}
	return _extends({}, state, {
		isSaving: false,
		savedData: action.data,
		data: [].concat(_toConsumableArray(state.data), [action.data])
	});
}

function saveError(state, action) {
	return _extends({}, state, {
		saveError: action.error,
		isSaving: false
	});
}

function deleting(state, action) {
	return _extends({}, state, {
		isDeleting: true,
		deleteError: deleteInitialState.error
	});
}

function deleted(state, action) {
	return _extends({}, state, {
		isDeleting: false
	});
}

function deletedWithMerge(state, action) {
	if (action.id) {
		var data = state.data.filter(function (item) {
			return item.id != action.id;
		});
		return _extends({}, state, {
			data: data,
			isDeleting: false
		});
	}
	return _extends({}, state, {
		isDeleting: false
	});
}

function deleteError(state, action) {
	return _extends({}, state, {
		deleteError: action.error,
		isDeleting: false
	});
}